namespace Eco.Mods.TechTree
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using Eco.Gameplay.Blocks;
    using Eco.Gameplay.Components;
    using Eco.Gameplay.Components.Auth;
    using Eco.Gameplay.DynamicValues;
    using Eco.Gameplay.Economy;
    using Eco.Gameplay.Housing;
    using Eco.Gameplay.Interactions;
    using Eco.Gameplay.Items;
    using Eco.Gameplay.Minimap;
    using Eco.Gameplay.Objects;
    using Eco.Gameplay.Players;
    using Eco.Gameplay.Property;
    using Eco.Gameplay.Skills;
    using Eco.Gameplay.Systems.TextLinks;
    using Eco.Gameplay.Pipes.LiquidComponents;
    using Eco.Gameplay.Pipes;
    using Eco.Gameplay.Pipes.Gases;
    using Eco.Gameplay.Systems.Tooltip;
    using Eco.Shared;
    using Eco.Shared.Math;
    using Eco.Shared.Localization;
    using Eco.Shared.Serialization;
    using Eco.Shared.Utils;
    using Eco.Shared.View;
    using Eco.Shared.Items;
    using Eco.World.Blocks;
    
    [Serialized]       
    [RequireComponent(typeof(CreditComponent))]
    [RequireComponent(typeof(CraftingComponent))]  
    [RequireComponent(typeof(AttachmentComponent))]
    [RequireComponent(typeof(PropertyAuthComponent))]
    [RequireComponent(typeof(SolidGroundComponent))]
    [RequireComponent(typeof(MustBeOwnedComponent))]
    public partial class TollObject : WorldObject
    {
        public override string FriendlyName { get { return "Toll Gate"; } }

        public CreditComponent Credit { get; private set; }

        private bool hasAnimation = false;
        private float timer = 0f;

        protected override void Initialize()
        {
            this.Credit = this.GetComponent<CreditComponent>();

            if (WorldObject.GetOccupancy(typeof(TollObject)) == null)
            WorldObject.AddOccupancy<TollObject>(new List<BlockOccupancy>(){
                new BlockOccupancy(new Vector3i(0, 0, 0), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 0, 1), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 0, 2), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 0, 3), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 0, 4), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 0, 5), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 0, 6), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),

                new BlockOccupancy(new Vector3i(0, 1, 0), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 1, 1), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 1, 2), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 1, 3), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 1, 4), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 1, 5), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 1, 6), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),

                new BlockOccupancy(new Vector3i(0, 2, 0), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 2, 1), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 2, 2), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 2, 3), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 2, 4), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 2, 5), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 2, 6), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),

                new BlockOccupancy(new Vector3i(0, 3, 0), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 3, 1), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 3, 2), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 3, 3), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 3, 4), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 3, 5), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 3, 6), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),

                new BlockOccupancy(new Vector3i(0, 4, 0), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 4, 1), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 4, 2), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 4, 3), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 4, 4), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 4, 5), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 4, 6), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),

                new BlockOccupancy(new Vector3i(0, 5, 0), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 5, 1), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 5, 2), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 5, 3), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 5, 4), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 5, 5), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
                new BlockOccupancy(new Vector3i(0, 5, 6), typeof(WorldObjectBlock), new Quaternion(0f, 0f, 0f, 1f)),
            });
        }

        protected override void PostInitialize()
        {
            this.GetComponent<PropertyAuthComponent>().Initialize(AuthModeType.Public);
        }

        public override InteractResult OnActRight(InteractionContext context)
        {
            if (context.Parameters != null && context.Parameters.ContainsKey("Open"))
            {
                if(hasAnimation && GetAnimatedState<bool>("Open"))
                    return InteractResult.NoOp;

                if (!this.Credit.CanPayFee(context.Player, 0, this.Credit.FeePerItem))
                    return InteractResult.NoOp;


                this.Credit.OrderComplete(context.Player, 0, this.Credit.FeePerItem, new LocString());
                SetAnimatedState("Open", true);
                hasAnimation = true;
                return InteractResult.Success;
            }
            return InteractResult.NoOp;
        }

        public override void Tick()
        {
            if(hasAnimation && GetAnimatedState<bool>("Open"))
            {
                if(timer >= ((this.Credit.FeePerMinute * 60) + 2))
                {
                    SetAnimatedState("Open", false);
                    timer = 0f;
                }
                timer += WorldObjectManager.TickDeltaTime;
            }
            base.Tick();
        }

        public override void Destroy()
        {
            base.Destroy();
        }
    }

    [Serialized]
    public partial class TollItem : WorldObjectItem<TollObject>
    {
        public override string FriendlyName { get { return "Toll Gate"; } } 
        public override string Description { get { return "Payment system for roads"; } }

        static TollItem()
        {
            
        }
        
    }

    [RequiresSkill(typeof(RoadConstructionSkill), 3)]
    public partial class TollRecipe : Recipe
    {
        public TollRecipe()
        {
            this.Products = new CraftingElement[]
            {
                new CraftingElement<TollItem>(),
            };

            this.Ingredients = new CraftingElement[]
            {
                new CraftingElement<IronIngotItem>(typeof(RoadConstructionEfficiencySkill), 10, RoadConstructionEfficiencySkill.MultiplicativeStrategy),
                new CraftingElement<ScrewsItem>(typeof(RoadConstructionEfficiencySkill), 5, RoadConstructionEfficiencySkill.MultiplicativeStrategy),
                new CraftingElement<GearItem>(typeof(RoadConstructionEfficiencySkill), 2, RoadConstructionEfficiencySkill.MultiplicativeStrategy),
            };
            this.CraftMinutes = CreateCraftTimeValue(typeof(TollRecipe), Item.Get<TollItem>().UILink(), 15, typeof(RoadConstructionSkill));;
            this.Initialize("Toll Gate", typeof(TollRecipe));
            CraftingComponent.AddRecipe(typeof(AssemblyLineObject), this);
        }
    }
}